#include "stdafx.hpp"

namespace
{
	class QueueDUI : public ui_element_instance, public CWindowImpl<QueueDUI>
	{
	public:
		QueueDUI(ui_element_config::ptr data, ui_element_instance_callback::ptr callback) : m_callback(callback)
		{
			set_configuration(data);
		}

		DECLARE_WND_CLASS_EX(L"QueueViewerWindowDUI", 0, -1);

		BEGIN_MSG_MAP_EX(QueueDUI)
			MSG_WM_CONTEXTMENU(on_context_menu);
			MSG_WM_CREATE(on_create);
			MSG_WM_SIZE(on_size);
		END_MSG_MAP()

		static GUID g_get_guid()
		{
			return guids::viewer_dui;
		}

		static GUID g_get_subclass()
		{
			return ui_element_subclass_utility;
		}

		static pfc::string8 g_get_description()
		{
			return "Playback Queue Viewer";
		}

		static ui_element_config::ptr g_get_default_configuration()
		{
			return ui_element_config::g_create_empty(g_get_guid());
		}

		static void g_get_name(pfc::string_base& out)
		{
			out = Component::name;
		}

		ui_element_config::ptr get_configuration() final
		{
			ui_element_config_builder builder;
			m_config.get(&builder.m_stream);
			return builder.finish(g_get_guid());
		}

		void initialize_window(HWND parent)
		{
			Create(parent);
		}

		void set_configuration(ui_element_config::ptr data) final
		{
			ui_element_config_parser parser(data);
			m_config.set(&parser.m_stream, parser.get_remaining());
		}

	protected:
		ui_element_instance_callback::ptr m_callback;

	private:
		int on_create(LPCREATESTRUCT)
		{
			m_list.CreateInDialog(*this, IDC_LIST);
			m_list.set_row_style(m_config.shade_alternate_rows);
			m_list.ui_fonts_changed();
			m_list.update_item_tf(m_config.item_tf);
			return 0;
		}

		void on_context_menu(CWindow, CPoint pt)
		{
			if (m_callback->is_edit_mode_enabled())
			{
				SetMsgHandled(FALSE);
			}
			else
			{
				m_list.on_context_menu(pt, m_config);
			}
		}

		void on_size(uint32_t, CSize size)
		{
			m_list.SetWindowPos(nullptr, 0, 0, size.cx, size.cy, SWP_NOZORDER);
		}

		QueueConfig m_config;
		QueueList m_list;
	};

	class UIElementImpl : public ui_element_impl_withpopup<QueueDUI> {};

	FB2K_SERVICE_FACTORY(UIElementImpl);
}
